﻿// m: Колебания по Бесселю (без реализаций)
// Версия 2.5 от 20.10.2015

window.addEventListener("load", main_BF, false);
function main_BF() {

    // Предварительные установки
    var a0 = 1;                             // масштаб расстояния
    var t0 = 1;                             // масштаб времени
    var m0 = 1;                             // масштаб массы

    var k0 = 2 * Math.PI / t0;              // масштаб частоты
    var C0 = m0 * k0 * k0;                  // масштаб жесткости

    var m = 1 * m0;                 	    // масса
    var C = 0.02 * C0;                 	    // жесткость

    var dx = 1 * a0;                        // шаг сетки по оси x
    var dt = 0.02 * t0;                     // шаг интегрирования по времени

    var N = 100000;                          // количество частиц в каждой реализации
    var periods = 10;                       // время расчета в периодах колебаний энергий

    var koeff = C / (dx * dx) / m * dt;     // коэффициент для уравнения динамики частиц

    var T;
    // функция переводит время в периодах в расчетное время
    function set_periods(n) {
        // первый раз кинетическая и потенциальная энергии доходят до нуля за 0.4 периода колебаний
        var T0 = Math.PI / (Math.sqrt(C / m)) / 2;  // период колебаний
        if (n >= 0.4 * T0) T = T0 * ( 0.4 + 0.75 + (n - 1) );
        else T = n;
    }
    set_periods(periods);

    // интерфейс программы
    N_number.value = N;
    N_number.oninput = function() {N = parseInt(N_number.value);};
    periods_number.value = periods;
    periods_number.oninput = function() {
        set_periods(parseFloat(periods_number.value));
    };
    button_start.onclick = calculate_new_system;
    checkbox_K.onchange = function() {
        draw();
        if (checkbox_K.checked) div_txt_K.style.display = "block";
        else div_txt_K.style.display = "none"
    };
    checkbox_P.onchange = function() {
        draw();
        if (checkbox_P.checked) div_txt_P.style.display = "block";
        else div_txt_P.style.display = "none"
    };
    checkbox_E.onchange = function() {
        draw();
        if (checkbox_E.checked) div_txt_E.style.display = "block";
        else div_txt_E.style.display = "none"
    };

    var ctx = BF_canvas.getContext("2d");   // на context происходит рисование
    var w = BF_canvas.width;                // ширина окна в расчетных координатах
    var h = BF_canvas.height;               // высота окна в расчетных координатах

    // основная расчетная функция
    var data_K, data_P, data_E;             // данные для графика
    var t, n;
    var t_start;
    function calculate_new_system() {
        t = 0;
        n = N + 2;                          // количество узлов по оси x + 2 для ГУ
        span_time_calc.innerHTML = "";
        div_calc_speed.style.display = "none";

        // задание начальных условий
        var P = [];
        for (var i = 1; i < n - 1; i++) {
            P[i] = {};
            P[i].u = 0;
            P[i].v = (2 * Math.random() - 1) * 0.25;
        }
        // периодические граничные условия
        P[0] = P[n - 2];
        P[n - 1] = P[1];

        data_K = []; data_P = []; data_E = [];

        // основной расчет
        t_start = performance.now();
        calculate(P);
    }
    calculate_new_system();

    function calculate(P) {
        var t1 = performance.now();
        while (t < T) {
            // расчет энергий для графиков
            var Kin = 0;
            var Pot = 0;
            for (var j = 1; j < n - 1; j++) {Kin += P[j].v * P[j].v;}
            for (var j = 1; j < n - 1; j++) {Pot += Math.pow(P[j].u - P[j - 1].u, 2);}
            Kin = Kin * m;
            Pot = Pot * C;
            data_K.push(Kin);
            data_P.push(Pot);
            data_E.push(Kin + Pot);

            // расчет состояния системы на следующем шаге
            for (var i = 1; i < n - 1; i++) {
                P[i].v += (P[i + 1].u - 2 * P[i].u + P[i - 1].u) * koeff;
            }
            for (var i = 1; i < n - 1; i++) {
                P[i].u += P[i].v * dt;
            }
            t += dt;

            // промежуточная прорисовка результатов, работа прогресс-бара
            if ((performance.now() - t1) > 200) {
                draw();
                setTimeout( function(){calculate(P); }, 10);
                div_container_progress.style.display = "block";
                button_start.style.display = "none";
                progress_bar.style.width = Math.round(t / T * 100) + "%";
                span_progress.innerHTML = (t / T * 100).toFixed(0) + "%";
                return;
            }
        }
        // окончательная прорисовка результатов, удаление прогресс-бара, вывод скорости расчета
        draw();
        div_container_progress.style.display = "none";
        button_start.style.display = "inline";
        div_calc_speed.style.display = "block";
        span_time_calc.innerHTML = parseFloat(((performance.now() - t_start) * 1000 * 1000 / N / Math.ceil(T/dt)).toPrecision(2));
        return true;
    }

    function draw() {
        ctx.clearRect(0, 0, w, h);              // очистка экрана

        if (checkbox_K.checked) {
            // Кинетическая энергия
            ctx.strokeStyle = "#ff0000";
            ctx.lineWidth = 1;
            ctx.beginPath();
            ctx.moveTo(0, 0);
            for (var i = 1; i < data_K.length; i++) {
                ctx.lineTo(i / (data_K.length - 1) * w, h - data_K[i] / data_K[0] * h);
            }
            ctx.stroke();
        }

        if (checkbox_P.checked) {
            // Потенциальная энергия
            ctx.strokeStyle = "#0000ff";
            ctx.lineWidth = 1;
            ctx.beginPath();
            ctx.moveTo(0, h - data_P[0] / data_K[0] * h);
            for (var i = 1; i < data_P.length; i++) {
                ctx.lineTo(i / (data_P.length - 1) * w, h - data_P[i] / data_K[0] * h);
            }
            ctx.stroke();
        }

        if (checkbox_E.checked) {
            // Полная энергия
            ctx.strokeStyle = "#880088";
            ctx.lineWidth = 2;
            ctx.beginPath();
            ctx.moveTo(0, h - data_E[0] / 2 / data_K[0] * h);
            for (var i = 1; i < data_E.length; i++) {
                ctx.lineTo(i / (data_E.length - 1) * w, h - data_E[i] / 2 / data_K[0] * h);
            }
            ctx.stroke();
        }
    }
}